//
// Copyright (C) 2005 Georg Lutz, Institut fuer Telematik
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#include "MfwRandomWP.h"

#define EV (ev.disabled()||!debug) ? ev : ev << logName() << "::MfwRandomWP: "

Define_Module( MfwRandomWP );

// do some initialization 
void MfwRandomWP::initialize(int stage)
{
  BasicMobility::initialize(stage); //DO NOT DELETE!!

	if(stage==0)
	{
	    updateInterval = par("updateInterval");
	    minSpeed = par("minSpeed");
	    maxSpeed = par("maxSpeed");
	    if(minSpeed == 0 && maxSpeed == 0)
		stationary = true;
	    else
		stationary = false;

	    timeToWait = 0;
	}
	else if(stage==1)
	{
	    if(!stationary)
	    {
		timeToUpdateMobility = new cMessage("timeToUpdateMobility");
		scheduleAt(simTime() + updateInterval,timeToUpdateMobility);
		timeToWait =  par("waitTime");
		timeToWait += simTime();
	    }
	}
}

void MfwRandomWP::handleSelfMsg(cMessage *msg)
{
    // are we waiting/pausing ?
    if(timeToWait != 0)
    {
	// timeToWait is over, we need a new target
	if(timeToWait - simTime() < updateInterval)
	{
	    setTargetPosition();
	    move();
	    timeToWait = 0;
	}
    }
    else
    {
	move();
	updatePosition();
	// have we reached target position?
	if(numSteps == step)
	{
	    timeToWait = par("waitTime");
	    timeToWait += simTime();
	}
    }
    scheduleAt(simTime() + updateInterval, timeToUpdateMobility);
}

void MfwRandomWP::setTargetPosition()
{
    targetPos=getRandomPosition();
    double distance=pos.distance(targetPos);
    double speed = uniform(minSpeed,maxSpeed);
    double totalTime = distance/speed;
    numSteps = FWMath::round(totalTime / updateInterval);

    stepSize.x = ((targetPos.x-pos.x) / numSteps);
    stepSize.y = ((targetPos.y-pos.y) / numSteps);

    step = 0;

    EV <<"distance="<<distance<<" xpos= "<<targetPos.x<<" ypos="<<targetPos.y
     <<"totalTime="<<totalTime<<" numSteps="<<numSteps<<" speed="<<speed<<endl;
}


void MfwRandomWP::move()
{
  step++;

  if( step <= numSteps ){
    EV <<"stepping forward. step #="<<step<<" xpos= "<<pos.x<<" ypos="<<pos.y<<endl;

    pos += stepSize;
  }
  else{
    EV <<"destination reached.\n xpos= "<<pos.x<<" ypos="<<pos.y<<endl;
    
    setTargetPosition();
  }
}
