//
// Copyright (C) 2005 Georg Lutz, Institut fuer Telematik
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/** @file GenRandomWP.h
    Random Waypoint Mobility for the Generic Model - header file.


    @author Institut fuer Telematik, Universitaet Karlsruhe, http://www.tm.uka.
de
    @author Georg Lutz (georglutz AT gmx DOT de)
    @date 2004-2005
*/
#ifndef MFW_RANDOMWP_MODULE_H
#define MFW_RANDOMWP_MODULE_H

#include <BasicMobility.h>

class MfwRandomWP : public BasicMobility
{
 protected:
    double updateInterval;
    cMessage *timeToUpdateMobility;

    double minSpeed;
    double maxSpeed;
    double waitTime;
    Coord targetPos; ///< the target position
    Coord stepSize; ///< stepsize of every move
    bool stationary; ///< am I fixed, i.e. access point / base station?
    double speed; ///< current speed of host
    double timeToWait; ///< wait/pause time
    int numSteps; ///< total step nr
    int step; ///< the actual step nr

    void setTargetPosition();
    void move();
    
 public:
  Module_Class_Members( MfwRandomWP , BasicMobility , 0 );

  /** @brief Initializes mobility model parameters.*/
  virtual void initialize(int);

 protected:
  /** @brief Called upon arrival of a self messages*/
  virtual void handleSelfMsg(cMessage *msg);

  // uncomment the following lines to use the blackboard
  /*
    virtual bool blackboardItemChanged( BBItemRef );
    virtual bool blackboardItemPublished( BBItemRef );
    virtual bool blackboardItemWithdrawn( BBItemRef );
  */
		
};

#endif

